@@ -2,7 +2,7 @@ use super::{
     imports::{ImportList, ImportType},
     ModuleError, ModulePreset, ModuleValidator,
 };
-use parity_wasm::elements::{External, FunctionType, ImportSection, Module, Type, ValueType};
+use parity_wasm::elements::{External, FunctionType, ImportSection, Module, Type};
 
 /// Enum representing the state of an import in a module.
 #[derive(PartialEq)]
@@ -36,25 +36,33 @@ pub struct VerifyImports<'a> {
 
 impl<'a> ModulePreset for VerifyImports<'a> {
     fn with_preset(preset: &str) -> Result<Self, ()> {
-        match preset {
-            "ewasm" => Ok(VerifyImports {
-                list: ImportList::with_preset("ewasm").unwrap(),
-                require_all: false,
-                allow_unlisted: false,
-            }),
-            _ => Err(()),
+        let mut import_set = ImportList::new();
+        let presets: String = preset
+            .chars()
+            .filter(|c| *c != ' ' && *c != '_' && *c != '\n' && *c != '\t')
+            .collect();
+
+        for preset_individual in presets.split(',') {
+            let to_append = ImportList::with_preset(preset_individual).expect("Invalid preset");
+            import_set.concatenate(to_append);
         }
+
+        Ok(VerifyImports {
+            list: import_set,
+            require_all: false, //FIXME: How should require_all and allow_unlisted be handled in the case of multiple presets?
+            allow_unlisted: false,
+        })
     }
 }
 
 // Utility functions used in tests to get more coverage
 #[cfg(test)]
 impl<'a> VerifyImports<'a> {
-    fn set_require_all(&mut self, arg: bool) {
+    pub fn set_require_all(&mut self, arg: bool) {
         self.require_all = arg;
     }
 
-    fn set_allow_unlisted(&mut self, arg: bool) {
+    pub fn set_allow_unlisted(&mut self, arg: bool) {
         self.allow_unlisted = arg;
     }
 }
@@ -97,7 +105,7 @@ impl<'a> ModuleValidator for VerifyImports<'a> {
                 .is_none(),
             (false, false) => {
                 // Check that all existent imports are listed and correct.
-                let mut checklist: Vec<ImportStatus> = self
+                let checklist: Vec<ImportStatus> = self
                     .list
                     .entries()
                     .iter()
@@ -269,7 +277,7 @@ fn has_func_import(module: &Module, namespace: &str, field: &str, sig: &Function
 
 /// Resolves an imported function's signature from its callable index.
 pub fn imported_func_sig_by_index(module: &Module, index: usize) -> FunctionType {
-    let import_section = module.import_section().expect("No function section found");
+    module.import_section().expect("No function section found");
     let type_section = module.type_section().expect("No type section found");
 
     match type_section.types()[index] {
@@ -280,7 +288,7 @@ pub fn imported_func_sig_by_index(module: &Module, index: usize) -> FunctionType
 #[cfg(test)]
 mod tests {
     use super::*;
-    use parity_wasm::elements::deserialize_buffer;
+    use parity_wasm::elements::{deserialize_buffer, ValueType};
 
     #[test]
     fn no_imports_ok_ewasm() {
